Package: querychat
Title: Filter and Query Data Frames in 'shiny' Using an LLM Chat
        Interface
Version: 0.3.0
Authors@R: c(
    person("Garrick", "Aden-Buie", , "garrick@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-7111-0077")),
    person("Joe", "Cheng", , "joe@posit.co", role = c("aut", "ccp")),
    person("Carson", "Sievert", , "carson@posit.co", role = "aut",
           comment = c(ORCID = "0000-0002-4958-2844")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Adds an LLM-powered chatbot to your 'shiny' app, that can
    turn your users' natural language questions into 'SQL' queries that
    run against your data, and return the result as a reactive data frame.
    Use it to drive reactive calculations, visualizations, downloads, and
    more.
License: MIT + file LICENSE
URL: https://posit-dev.github.io/querychat/r/,
        https://posit-dev.github.io/querychat/,
        https://github.com/posit-dev/querychat
BugReports: https://github.com/posit-dev/querychat/issues
Depends: R (>= 4.1.0)
Imports: bsicons, bslib (>= 0.11.0), cli, DBI, ellmer (>= 0.4.1),
        htmltools, lifecycle, promises, R6, rlang (>= 1.1.0), S7,
        shiny, shinychat (>= 0.4.0), utils, whisker
Suggests: dbplyr, dplyr, DT, duckdb, ggsql, knitr, palmerpenguins,
        rmarkdown, RSQLite, rsvg, shinytest2, testthat (>= 3.0.0), V8,
        withr
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-06-01 20:54:39 UTC; cpsievert
Author: Garrick Aden-Buie [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7111-0077>),
  Joe Cheng [aut, ccp],
  Carson Sievert [aut] (ORCID: <https://orcid.org/0000-0002-4958-2844>),
  Posit Software, PBC [cph, fnd]
Maintainer: Garrick Aden-Buie <garrick@posit.co>
Repository: CRAN
Date/Publication: 2026-06-01 22:00:02 UTC
Built: R 4.5.2; ; 2026-06-02 00:59:41 UTC; unix
