% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{deprecated}
\alias{deprecated}
\alias{querychat_init}
\alias{querychat_sidebar}
\alias{querychat_ui}
\alias{querychat_server}
\alias{querychat_greeting}
\alias{querychat_data_source}
\title{Deprecated functions}
\usage{
querychat_init(...)

querychat_sidebar(...)

querychat_ui(...)

querychat_server(...)

querychat_greeting(...)

querychat_data_source(...)
}
\value{
Please see the updated function details for return values.
}
\description{
These functions have been replaced by the new \code{QueryChat} R6 class API.
Please update your code to use the new class-based approach.
}
\section{Functions}{
\itemize{
\item \code{querychat_init()}: was replaced with the \verb{$new()} method of \link{QueryChat}.

\item \code{querychat_sidebar()}: was replaced with the \verb{$sidebar()} method of
\link{QueryChat}.

\item \code{querychat_ui()}: was replaced with the \verb{$ui()} method of \link{QueryChat}.

\item \code{querychat_server()}: was replaced with the \verb{$server()} method of
\link{QueryChat}.

\item \code{querychat_greeting()}: was replaced with the \verb{$generate_greeting()} method of
\link{QueryChat}.

\item \code{querychat_data_source()}: was replaced with the \verb{$new()} method of \link{QueryChat}.

}}
\keyword{internal}
